//Funciones para la gestión del puerto serie


// Esta función maneja eventos de recepción de datos en el puerto serial. 
// Lee datos desde el puerto serial hasta que se encuentre el carácter de salto de línea ('\n') o se llene el búfer ackBuff.
// Verifica si se han leído al menos 4 bytes (numBytesRead >= 4) y si los últimos 4 bytes leídos son 'A', 'C', 'K'. entonces se pasa al el estado reqStatus a requestStatus_t.ARRIVING
// Si se da la condición anterior se cambia el estado reqStatus a requestStatus_t.ARRIVING.

void serialEvent(Serial serialPort) {

  if (reqStatus == requestStatus_t.REQUESTED) {
    int numBytesRead = serialPort.readBytesUntil(G_DEF.LF, ackBuff);
    if ((numBytesRead >= 4) && 
      (ackBuff[numBytesRead-4] == 'A') &&
      (ackBuff[numBytesRead-3] == 'C') &&
      (ackBuff[numBytesRead-2] == 'K') ) {
      reqStatus = requestStatus_t.ARRIVING;

      println(ackBuff[0]);
    } else if (millis() > waitTimeout) {
      reqStatus = requestStatus_t.TIMEOUT;
    }
  } else if (reqStatus == requestStatus_t.ARRIVING) {
    parseSerialData();
  } else {
    if (bSerialDebug) print(serialPort.readStringUntil(G_DEF.LF));
  }
}

//Esta función se encarga de analizar los datos recibidos a través del puerto serie.

void parseSerialData() {
  switch (request) {
  case NONE:         
    break;
  case TRACKDARK: 
  case TRACKBRIG:     
    tmp_x0 = int(serialPort.read());
    tmp_y0 = int(serialPort.read());
    tmp_x1 = int(serialPort.read());
    tmp_y1 = int(serialPort.read());
    reqStatus = requestStatus_t.RECEIVED;
    break;
  case STREAM0PPB:
  case STREAM1PPB:
  case STREAM2PPB:
  case STREAM4PPB:
  case STREAM8PPB:   
    serialPort.readBytes(pix[currRow]);
    serialPort.clear();
    currRow++;
    if (currRow >= G_DEF.F_H) {
      reqStatus = requestStatus_t.RECEIVED; // Fotograma listo en el búfer
      currRow = 0;
      if (bSerialDebug) println();
    }
    break;
  default :       
    break;
  }
}

// funcion para obtener el valor ASCII que abrirá cada una de las pantallas
int getValor() {

  if (ackBuff[0]==52 || ackBuff[0]==53 || ackBuff[0]==113 || ackBuff[0]==54 || ackBuff[0]==119 || ackBuff[0]==55 || ackBuff[0]==101 || ackBuff[0]==51 || ackBuff[0]==114 || ackBuff[0]==57 || ackBuff[0]==116 || ackBuff[0]==50 || ackBuff[0]==121 || ackBuff[0]==99 || ackBuff[0]==118) {

    valor=ackBuff[0];
  }
  return valor;
}
