/* 
 Clase para la definición de las variables globales del sistema
 SCR_W y SCR_H: Representan el ancho y alto de la pantalla en píxeles.
 FONT_SIZE: Define el tamaño de la fuente
 FONT_BKG_SIZE: Define el tamaño del fondo de la fuente
 DRAW_SCALE: Representa la escala de dibujo   
 MAX_ROW_LEN: Define la longitud máxima de una fila en píxeles  
 MAX_ROW_BUFF_LEN: Define la longitud máxima de un búfer de fila, que incluye los píxeles y el carácter LF
 ACK_BUF_LEN: Define la longitud máxima de un búfer de respuesta para almacenar "ACK" y LF (salto de línea)
 SERIAL_TIMEOUT: Define el tiempo máximo de espera en milisegundos para recibir una respuesta (ACK).
 */
final class G_DEF {
  public final static int BAUDRATE = 500000;

  public final static char  LF           = '\n';    // Salto de línea en ASCII
  public final static char  CR           = '\r';    // Retorno de carro en ASCII

  public final static int   SCR_W        = 650;
  public final static int   SCR_H        =700;
  public final static int   F_W          = 80;
  public final static int   F_H          = 60;
  public final static int   BPP          = 2;
  public final static int   FONT_SIZE    = 18;
  public final static int   FONT_BKG_SIZE  = (int)(FONT_SIZE*(float)1.5);
  public final static float DRAW_SCALE   = (float)SCR_W / (float)F_W;

  public final static int   MAX_ROW_LEN      = F_W*BPP; // pixeles + LF (caracter de salto de línea)
  public final static int   MAX_ROW_BUFF_LEN = MAX_ROW_LEN+1; // pixeles + LF (caracter de salto de línea)
  public final static int   ACK_BUF_LEN      = 255; // búfer serial para almacenar "ACK"+LF
  public final static long  SERIAL_TIMEOUT   = 2; // milisegundos para esperar ACK
}


/*
 Enumeración con los siguientes elementos:
 NONE(0): Representa una solicitud sin ningún parámetro.
 TRACKDARK(1): Representa una solicitud de seguimiento en condiciones de poca luz.
 TRACKBRIG(2): Representa una solicitud de seguimiento en condiciones de mucha luz.
 STREAM8PPB(G_DEF.F_W/8): Representa una solicitud de transmisión de datos con una resolución de 8 píxeles por byte.
 STREAM4PPB(G_DEF.F_W/4): Representa una solicitud de transmisión de datos con una resolución de 4 píxeles por byte.
 STREAM2PPB(G_DEF.F_W/2): Representa una solicitud de transmisión de datos con una resolución de 2 píxeles por byte.
 STREAM1PPB(G_DEF.F_W): Representa una solicitud de transmisión de datos con una resolución de 1 píxel por byte
 STREAM0PPB(G_DEF.F_W*G_DEF.BPP): Representa una solicitud de transmisión de datos sin compresión.
 */

enum requestStatus_t {
  IDLE, REQUESTED, TIMEOUT, ARRIVING, RECEIVED, PROCESSING
};

enum request_t {
  NONE(0), 
    TRACKDARK(1), 
    TRACKBRIG(2), 
    STREAM8PPB(G_DEF.F_W/8), 
    STREAM4PPB(G_DEF.F_W/4), 
    STREAM2PPB(G_DEF.F_W/2), 
    STREAM1PPB(G_DEF.F_W), 
    STREAM0PPB(G_DEF.F_W*G_DEF.BPP);

  private int value;    

  private request_t(int value) {
    this.value = value;
  }

  public int getParam() {
    return value;
  }
};
