// Funciones para la visualización del vídeo obtenido por la cámara

void drawVideo() {

  // Dibujo del vídeo de los visitantes
  switch (request) {
  case NONE:        
    reqStatus = requestStatus_t.IDLE;
    background(0);
    // drawInfo();
    break;
  case TRACKDARK: 
  case TRACKBRIG:    
    switch (reqStatus) {
    case RECEIVED:  
      //   drawTracking();
      //   drawFPS();
      reqStatus = requestStatus_t.IDLE;
      break;
    case TIMEOUT:   
    case IDLE:      
      reqTracking(request);
      reqStatus = requestStatus_t.REQUESTED; 
      waitTimeout = G_DEF.SERIAL_TIMEOUT + millis();
      break;
    case REQUESTED:  
      if (millis() > waitTimeout) reqStatus = requestStatus_t.TIMEOUT;
      break;
    case ARRIVING:  
      break;
    default : 
      break;
    }

    //   drawInfo();
    break;
  case STREAM0PPB:
  case STREAM1PPB:
  case STREAM2PPB:
  case STREAM4PPB:
  case STREAM8PPB:  
    switch (reqStatus) {
    case RECEIVED:  
      reqStatus = requestStatus_t.PROCESSING;
      buff2pixFrame(pix, currFrame, request);
      noSmooth();
      image(currFrame, 155, 200, 670, 670);
      smooth();
      //  drawFPS();
      reqStatus = requestStatus_t.IDLE;
      break;
    case TIMEOUT:   
    case IDLE:      
      reqImage(request);
      reqStatus = requestStatus_t.REQUESTED; 
      waitTimeout = G_DEF.SERIAL_TIMEOUT + millis();
      break;
    case REQUESTED:  
      if (millis() > waitTimeout) reqStatus = requestStatus_t.TIMEOUT;
      break;
    case ARRIVING:  
      break;
    case PROCESSING:  
      break;
    default : 
      break;
    }   
    //  drawInfo();
    break;
  default :        
    break;
  }
}


// Solicitud de imagen
void reqImage(request_t req) {
  currRow = 0;
  serialPort.clear();
  serialPort.write("thresh "+Integer.toString(int(thresh))+G_DEF.LF);
  serialPort.write("send "+Integer.toString(req.getParam())+G_DEF.LF);
}


// Solicitud de datos de seguimiento
void reqTracking(request_t req) {

  serialPort.clear();
  if (req == request_t.TRACKDARK)
    serialPort.write("dark "+Integer.toString(int(thresh))+G_DEF.LF);
  else if (req == request_t.TRACKBRIG)
    serialPort.write("brig "+Integer.toString(int(thresh))+G_DEF.LF);
}

// Dibuja la información de la pantalla
void drawInfo() {
  String modeStr = "Press space to toggle MODE: "+request;
  pushStyle();
  pushMatrix();
  noStroke();
  fill(64);
  rect(0, height-G_DEF.FONT_BKG_SIZE, width, G_DEF.FONT_BKG_SIZE);
  fill(255);
  textAlign(LEFT, TOP);
  text(modeStr, 20, height-G_DEF.FONT_BKG_SIZE);
  textAlign(RIGHT, TOP);
  text("thresh (+/-): "+int(thresh), width-20, height-G_DEF.FONT_BKG_SIZE);
  popMatrix();
  popStyle();
}


// Dibuja los FPS
void drawFPS() {
  long currTime = millis();
  float fps =1000.0/(float)(currTime-fpsTimeStamp);

  pushStyle();
  pushMatrix();
  noStroke();
  fill(0);
  rect(20, 20, textWidth("FPS: "+fps), G_DEF.FONT_SIZE);
  fill(255);
  translate(0, -2);
  textAlign(LEFT, TOP);
  text("FPS: "+fps, 20, 20);
  popMatrix();
  popStyle();
  fpsTimeStamp = currTime;
}


//Dibujo del marco par el visionado del vídeo
void drawTracking() {
  pushStyle();
  noFill(); 
  strokeWeight(3);

  if (tmp_x1-tmp_x0 > 3 && tmp_x1-tmp_x0 < G_DEF.F_W/2 && tmp_y1-tmp_y0 < G_DEF.F_H/2 && tmp_y1-tmp_y0 > 3) {
    stroke(0);  
    float minX = width-x0*G_DEF.DRAW_SCALE; 
    float minY = y0*G_DEF.DRAW_SCALE;
    float wX = -(x1-x0)*G_DEF.DRAW_SCALE;
    float hY = (y1-y0)*G_DEF.DRAW_SCALE;        
    float centX = minX+wX/2;
    float centY = minY+hY/2;
    rect(minX, minY, wX, hY );
    line( centX-10, centY, centX+10, centY );
    line( centX, centY-10, centX, centY+10 );
    if (bKalmanEnabled) {
      x0 = (float)filter_x0.update(tmp_x0);
      y0 = (float)filter_y0.update(tmp_y0);
      x1 = (float)filter_x1.update(tmp_x1);
      y1 = (float)filter_y1.update(tmp_y1);
    } else {
      x0 = tmp_x0;
      y0 = tmp_y0;
      x1 = tmp_x1;
      y1 = tmp_y1;
    }
    stroke(255, 0, 0); 
    minX = width-x0*G_DEF.DRAW_SCALE; 
    minY = y0*G_DEF.DRAW_SCALE;
    wX = -(x1-x0)*G_DEF.DRAW_SCALE;
    hY = (y1-y0)*G_DEF.DRAW_SCALE;        
    centX = minX+wX/2;
    centY = minY+hY/2;

    rect(minX, minY, wX, hY );


    line( centX-10, centY, centX+10, centY );
    line( centX, centY-10, centX, centY+10 );
    stroke(0, 0, 255);
    line(lastCenter.x, lastCenter.y, centX, centY);
    lastCenter.x = centX;
    lastCenter.y = centY;
  }
  popStyle();
}


// Conversión de los pixeles recibido en el buffer a PImage
// Contempla los diferentes formatos de imgen dependiendo de la resolución escogida (pixeles por byte)
void buff2pixFrame(byte[][] pixBuff, PImage dstImg, request_t req) {

  int Y0 = 0, U = 0, Y1 = 0, V = 0;
  int Y2 = 0, Y3 = 0, Y4 = 0;
  int reqParam = req.getParam();

  dstImg.loadPixels();

  switch (req) {
  case STREAM0PPB: 
    for (int y = 0, l = 0, x = 0; y < G_DEF.F_H; y++, x = 0)
      while (x < reqParam) {
        Y0 = int(pixBuff[y][x++]);
        U  = int(pixBuff[y][x++]);
        Y1 = int(pixBuff[y][x++]);
        V  = int(pixBuff[y][x++]);
        dstImg.pixels[l++] = YUV2RGB(Y0, U, V);
        dstImg.pixels[l++] = YUV2RGB(Y1, U, V);
      }
    break;
  case STREAM1PPB: 
    for (int y = 0, l = 0, x = 0; y < G_DEF.F_H; y++, x = 0)
      while (x < reqParam) {
        Y0 = 0x08 | ((0x0f & int(pixBuff[y][x])) << 4);
        U  = 0x00 | ((0xf0 & int(pixBuff[y][x++])));
        Y1 = 0x08 | ((0x0f & int(pixBuff[y][x])) << 4);
        V  = 0x00 | ((0xf0 & int(pixBuff[y][x++])));
        dstImg.pixels[l++] = YUV2RGB(Y0, U, V);
        dstImg.pixels[l++] = YUV2RGB(Y1, U, V);
      }
    break;
  case STREAM2PPB:  
    for (int y = 0, l = 0, x = 0; y < G_DEF.F_H; y++, x = 0)
      while (x < reqParam) {
        Y1 = int(pixBuff[y][x++]);
        Y0 = ((Y1 << 4) & 0xF0) | 0x1f;
        Y1 = (Y1 & 0xF0)  | 0x1f;
        dstImg.pixels[l++] = color(Y0);
        dstImg.pixels[l++] = color(Y1);
      }
    break;
  case STREAM4PPB:  
    for (int y = 0, l = 0, x = 0; y < G_DEF.F_H; y++, x = 0)
      while (x < reqParam) {
        Y3 = int(pixBuff[y][x++]);
        Y0 = ((Y3 << 6) & 0xC0) | 0x1f;
        Y1 = ((Y3 << 4) & 0xC0) | 0x1f;
        Y2 = ((Y3 << 2) & 0xC0) | 0x1f;
        Y3 = (Y3 & 0xC0) | 0x1f;
        dstImg.pixels[l++] = color(Y0);
        dstImg.pixels[l++] = color(Y1);
        dstImg.pixels[l++] = color(Y2);
        dstImg.pixels[l++] = color(Y3);
      }
    break;
  case STREAM8PPB:  
    for (int y = 0, l = 0, x = 0; y < G_DEF.F_H; y++, x = 0)
      while (x < reqParam) {
        Y0 = int(pixBuff[y][x++]);

        dstImg.pixels[l++] = ((Y0 & 0x01) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x02) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x04) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x05) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x10) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x20) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x40) == 0? 0:0xffffff);
        dstImg.pixels[l++] = ((Y0 & 0x80) == 0? 0:0xffffff);
      }
    break;
  }
  dstImg.updatePixels();
}


//Conversión de YUV a RGB
color YUV2RGB(int Y, int Cb, int Cr) {
  // from OV7670 Software Application note
  float R = Y + 1.371*(Cr-128);
  float G = Y - 0.698*(Cr-128)+0.336*(Cb-128);
  float B = Y + 1.732*(Cb-128);

  return color(R, G, B);
}

//Conversión RGB a YUV
color RGB2YUV(int R, int G, int B) {
  // from OV7670 Software Application note
  float Yu = 0.299*R+0.587*G+0.114*B;
  float Cb = 0.568*(B-Y)+128;
  float Cr = 0.713*(R-Y)+128;

  return color(Yu, Cb, Cr);
}
